#!/usr/bin/env python

# SVGAni - A simple, manual SVG animation tool
# Version 1.0, 2009-05-21
#
# Usage: ./svgani.py svgani.settings
#
# For details about the necessary modifications to the SVG input file, and
# the format of the svgani.settings file, please visit:
# http://www.mbeckler.org/inkscape/svgani/
#
# Written by Matthew L Beckler - matthew at mbeckler dot org
#
#
# SVGAni is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# SVGAni is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with SVGAni (the file COPYING.txt); if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import ConfigParser
import commands
import sys, os, time

# This is an empty class that we use as a struct
class struct: pass

def svgani():
    if len(sys.argv) != 2:
        print "Usage: " + sys.argv[0] + " svgani.settings"
        sys.exit(1)

    filename = sys.argv[1]

    config = ConfigParser.RawConfigParser()
    config.read(filename)
    
    # read in our general configuration information
    input_svg_filename = config.get('general', 'input_svg_filename')
    output_png_basename = config.get('general', 'output_png_basename')
    output_width = config.getint('general', 'output_width')
    output_height = config.getint('general', 'output_height')
    convert_to_gif = config.getboolean('general', 'convert_to_gif')
    num_phases = config.getint('general', 'num_phases')
    draw_progress = config.getboolean('general', 'draw_progress')
    modules = config.get('general', 'modules')

    # import all the extra modules required by the settings file
    module_list = modules.split()
    sys.stdout.write("Importing modules:")
    for module in module_list:
        sys.stdout.write(" " + module)
        sys.stdout.flush()
        try:
            vars()[module] = __import__(module)
        except:
            print ""
            print "Could not import module: '%s', exiting" % module
            sys.exit(1)
    sys.stdout.write("\n")

    # read in the data for each phase
    phase_info = []
    for p in range(0, num_phases):
        section = "phase_%04d" % p
        try:
            index_start = config.getfloat(section, 'index_start')
            index_end = config.getfloat(section, 'index_end')
            steps = config.getint(section, 'steps')
        except:
            print "You are missing one of 'index_start', 'index_end', 'steps' from section '%s', exiting" % section
            sys.exit(1)
        repl_list = []
        for name, value in config.items(section):
            # All name:value pairs that are not index_start, index_end, or steps
            # are search and replace pairs
            if name not in ('index_start', 'index_end', 'steps'):
                repl_list.append( (name, value) )
        if len(repl_list) < 1:
            print("You probably want to include at least one replacement string in section '%s', exiting" % section)
            sys.exit(1)
        info = struct()
        info.phase_num = p
        info.index_start = index_start
        info.index_end = index_end
        info.steps = steps
        info.repl_list = repl_list
        phase_info.append( info )

    # read in our base svg file to the svg_data variable
    try:
        svg_file = open(input_svg_filename, 'r')
        svg_data = svg_file.read()
        svg_file.close()
    except:
        print("Could not open and read in the svg file '%s', exiting" % input_svg_filename)
        sys.exit(1)

    total_steps = 0
    for info in phase_info:
        total_steps = total_steps + info.steps
    print "Finished parsing configuration file."
    print "There are %d phases with %d total frames." % (len(phase_info), total_steps)

    global_ix = 0 # used for writing filenames
    for info in phase_info:
        print "Rendering phase: %4d (%4d steps)" % (info.phase_num, info.steps)
        for ix in range(0, info.steps):
            if draw_progress:
                (term_rows, term_cols) = commands.getoutput("stty size").split()
                term_rows = int(term_rows)
                term_cols = int(term_cols)
                pct = int( (global_ix / float(total_steps)) * 100)
                num_stars = int((term_cols - 7) * (global_ix / float(total_steps)))
                stars = "*" * num_stars
                spaces = " " * (term_cols - num_stars - 7)
                sys.stdout.write("%3d%% [%s%s]" % (pct, stars, spaces))
                sys.stdout.flush()

            # define our index variable, evaluate all the search/replace pairs, and write temp.svg file
            index = (ix/float(info.steps-1) * (info.index_end - info.index_start) ) + info.index_start
            temp = svg_data
            for repl in info.repl_list:
                temp = temp.replace(repl[0], eval(repl[1]))
            ofile = open("temp.svg", 'w')
            ofile.write(temp)
            ofile.close()

            # convert temp.svg to temp.png using inkscape
            inkscape_cmd = "inkscape --without-gui --file=temp.svg --export-png=temp.png --export-area-canvas --export-width=%d --export-height=%d" % (output_width, output_height)
            (inkscape_status, inkscape_output) = commands.getstatusoutput(inkscape_cmd)

            if (inkscape_status != 0) or (not os.path.exists("temp.png")):
                print "Error: 'temp.png' was not found or there was a non-zero exit code from Inkscape. There was most likely a problem with inkscape when we tried to execute this command:"
                print inkscape_cmd
                print "Inkscape output:"
                print inkscape_output
                sys.exit(1)

            os.remove("temp.svg")

            # either convert to gif, or rename to the proper png filename
            obasename = "%s%04d" % (output_png_basename, global_ix)
            if convert_to_gif:
                # convert temp.png to the proper gif filename
                convert_cmd = "convert temp.png %s.gif" % obasename
                (convert_status, convert_output) = commands.getstatusoutput(convert_cmd)
                if (convert_status != 0) or (not os.path.exists("%s.gif" % obasename)):
                    print "Error: '%s.gif' was not found, or there was a non-zero exit code from 'convert'." % obasename
                    print "Command used:"
                    print convert_cmd
                    print "Convert output:"
                    print convert_output
                    sys.exit(1)
                os.remove("temp.png")
            else:
                # rename temp.png to the proper png filename
                ofilename = obasename + ".png"
                os.rename('temp.png', obasename)

            global_ix = global_ix + 1
            if draw_progress:
                sys.stdout.write("%s[2K" % chr(27))
                sys.stdout.write("%s[%dD" % (chr(27), term_cols))
                sys.stdout.flush()
    print "Finished!"


if __name__=="__main__":
    svgani()

